@@ -18,10 +18,9 @@ sub _carp {
     return warn @_, " at $file line $line\n";
 }
 
-our $VERSION = '1.001014';
-$VERSION = eval $VERSION;    ## no critic (BuiltinFunctions::ProhibitStringyEval)
+our $VERSION = '1.302191';
 
-use Test::Builder::Module 0.99;
+use Test::Builder::Module;
 our @ISA    = qw(Test::Builder::Module);
 our @EXPORT = qw(ok use_ok require_ok
   is isnt like unlike is_deeply
@@ -38,7 +37,7 @@ our @EXPORT = qw(ok use_ok require_ok
   BAIL_OUT
 );
 
-#line 163
+#line 166
 
 sub plan {
     my $tb = Test::More->builder;
@@ -54,12 +53,22 @@ sub import_extra {
 
     my @other = ();
     my $idx   = 0;
+    my $import;
     while( $idx <= $#{$list} ) {
         my $item = $list->[$idx];
 
         if( defined $item and $item eq 'no_diag' ) {
             $class->builder->no_diag(1);
         }
+        elsif( defined $item and $item eq 'import' ) {
+            if ($import) {
+                push @$import, @{$list->[ ++$idx ]};
+            }
+            else {
+                $import = $list->[ ++$idx ];
+                push @other, $item, $import;
+            }
+        }
         else {
             push @other, $item;
         }
@@ -69,17 +78,29 @@ sub import_extra {
 
     @$list = @other;
 
+    if ($class eq __PACKAGE__ && (!$import || grep $_ eq '$TODO', @$import)) {
+        my $to = $class->builder->exported_to;
+        no strict 'refs';
+        *{"$to\::TODO"} = \our $TODO;
+        if ($import) {
+            @$import = grep $_ ne '$TODO', @$import;
+        }
+        else {
+            push @$list, import => [grep $_ ne '$TODO', @EXPORT];
+        }
+    }
+
     return;
 }
 
-#line 216
+#line 245
 
 sub done_testing {
     my $tb = Test::More->builder;
     $tb->done_testing(@_);
 }
 
-#line 288
+#line 317
 
 sub ok ($;$) {
     my( $test, $name ) = @_;
@@ -88,7 +109,7 @@ sub ok ($;$) {
     return $tb->ok( $test, $name );
 }
 
-#line 371
+#line 400
 
 sub is ($$;$) {
     my $tb = Test::More->builder;
@@ -102,10 +123,10 @@ sub isnt ($$;$) {
     return $tb->isnt_eq(@_);
 }
 
-*isn't = \&isnt;
-# ' to unconfuse syntax higlighters
+# make this available as isn't()
+*isn::t = \&isnt;
 
-#line 416
+#line 445
 
 sub like ($$;$) {
     my $tb = Test::More->builder;
@@ -113,7 +134,7 @@ sub like ($$;$) {
     return $tb->like(@_);
 }
 
-#line 431
+#line 460
 
 sub unlike ($$;$) {
     my $tb = Test::More->builder;
@@ -121,7 +142,7 @@ sub unlike ($$;$) {
     return $tb->unlike(@_);
 }
 
-#line 477
+#line 506
 
 sub cmp_ok($$$;$) {
     my $tb = Test::More->builder;
@@ -129,7 +150,7 @@ sub cmp_ok($$$;$) {
     return $tb->cmp_ok(@_);
 }
 
-#line 512
+#line 541
 
 sub can_ok ($@) {
     my( $proto, @methods ) = @_;
@@ -163,7 +184,7 @@ sub can_ok ($@) {
     return $ok;
 }
 
-#line 578
+#line 607
 
 sub isa_ok ($$;$) {
     my( $thing, $class, $thing_name ) = @_;
@@ -245,7 +266,7 @@ WHOA
     return $ok;
 }
 
-#line 679
+#line 708
 
 sub new_ok {
     my $tb = Test::More->builder;
@@ -270,16 +291,14 @@ sub new_ok {
     return $obj;
 }
 
-#line 765
+#line 805
 
 sub subtest {
-    my ($name, $subtests) = @_;
-
     my $tb = Test::More->builder;
     return $tb->subtest(@_);
 }
 
-#line 789
+#line 827
 
 sub pass (;$) {
     my $tb = Test::More->builder;
@@ -293,7 +312,7 @@ sub fail (;$) {
     return $tb->ok( 0, @_ );
 }
 
-#line 842
+#line 880
 
 sub require_ok ($) {
     my($module) = shift;
@@ -338,14 +357,17 @@ sub _is_module_name {
 }
 
 
-#line 936
+#line 974
 
 sub use_ok ($;@) {
     my( $module, @imports ) = @_;
     @imports = () unless @imports;
     my $tb = Test::More->builder;
 
-    my( $pack, $filename, $line ) = caller;
+    my %caller;
+    @caller{qw/pack file line sub args want eval req strict warn/} = caller(0);
+
+    my ($pack, $filename, $line, $warn) = @caller{qw/pack file line warn/};
     $filename =~ y/\n\r/_/; # so it doesn't run off the "#line $line $f" line
 
     my $code;
@@ -354,7 +376,7 @@ sub use_ok ($;@) {
         # for it to work with non-Exporter based modules.
         $code = <<USE;
 package $pack;
-
+BEGIN { \${^WARNING_BITS} = \$args[-1] if defined \$args[-1] }
 #line $line $filename
 use $module $imports[0];
 1;
@@ -363,14 +385,14 @@ USE
     else {
         $code = <<USE;
 package $pack;
-
+BEGIN { \${^WARNING_BITS} = \$args[-1] if defined \$args[-1] }
 #line $line $filename
 use $module \@{\$args[0]};
 1;
 USE
     }
 
-    my( $eval_result, $eval_error ) = _eval( $code, \@imports );
+    my ($eval_result, $eval_error) = _eval($code, \@imports, $warn);
     my $ok = $tb->ok( $eval_result, "use $module;" );
 
     unless($ok) {
@@ -406,7 +428,7 @@ sub _eval {
 }
 
 
-#line 1037
+#line 1092
 
 our( @Data_Stack, %Refs_Seen );
 my $DNE = bless [], 'Does::Not::Exist';
@@ -506,14 +528,14 @@ sub _type {
 
     return '' if !ref $thing;
 
-    for my $type (qw(Regexp ARRAY HASH REF SCALAR GLOB CODE)) {
+    for my $type (qw(Regexp ARRAY HASH REF SCALAR GLOB CODE VSTRING)) {
         return $type if UNIVERSAL::isa( $thing, $type );
     }
 
     return '';
 }
 
-#line 1197
+#line 1252
 
 sub diag {
     return Test::More->builder->diag(@_);
@@ -523,23 +545,26 @@ sub note {
     return Test::More->builder->note(@_);
 }
 
-#line 1223
+#line 1278
 
 sub explain {
     return Test::More->builder->explain(@_);
 }
 
-#line 1289
+#line 1344
 
 ## no critic (Subroutines::RequireFinalReturn)
 sub skip {
     my( $why, $how_many ) = @_;
     my $tb = Test::More->builder;
 
-    unless( defined $how_many ) {
-        # $how_many can only be avoided when no_plan is in use.
+    # If the plan is set, and is static, then skip needs a count. If the plan
+    # is 'no_plan' we are fine. As well if plan is undefined then we are
+    # waiting for done_testing.
+    unless (defined $how_many) {
+        my $plan = $tb->has_plan;
         _carp "skip() needs to know \$how_many tests are in the block"
-          unless $tb->has_plan eq 'no_plan';
+            if $plan && $plan =~ m/^\d+$/;
         $how_many = 1;
     }
 
@@ -557,7 +582,7 @@ sub skip {
     last SKIP;
 }
 
-#line 1373
+#line 1440
 
 sub todo_skip {
     my( $why, $how_many ) = @_;
@@ -578,7 +603,7 @@ sub todo_skip {
     last TODO;
 }
 
-#line 1428
+#line 1495
 
 sub BAIL_OUT {
     my $reason = shift;
@@ -587,7 +612,7 @@ sub BAIL_OUT {
     $tb->BAIL_OUT($reason);
 }
 
-#line 1467
+#line 1534
 
 #'#
 sub eq_array {
@@ -727,7 +752,7 @@ WHOA
     }
 }
 
-#line 1614
+#line 1681
 
 sub eq_hash {
     local @Data_Stack = ();
@@ -762,7 +787,7 @@ sub _eq_hash {
     return $ok;
 }
 
-#line 1673
+#line 1740
 
 sub eq_set {
     my( $a1, $a2 ) = @_;
@@ -787,6 +812,6 @@ sub eq_set {
     );
 }
 
-#line 1946
+#line 2004
 
 1;
