@@ -3,7 +3,7 @@ package Test::Nginx::Util;
 use strict;
 use warnings;
 
-our $VERSION = '0.29';
+our $VERSION = '0.32';
 
 use base 'Exporter';
 
@@ -20,10 +20,12 @@ use Scalar::Util qw( looks_like_number );
 use IO::Socket::INET;
 use IO::Socket::UNIX;
 use Test::LongString;
+use POSIX ":sys_wait_h";
 use Carp qw( croak );
 
 our $ConfigVersion;
 our $FilterHttpConfig;
+our $RandPorts;
 my $LoadedIPCRun;
 
 our $NoLongString = undef;
@@ -31,6 +33,8 @@ our $FirstTime = 1;
 
 our $ReusePort = $ENV{TEST_NGINX_REUSE_PORT};
 
+our $UseHttp3 = $ENV{TEST_NGINX_USE_HTTP3};
+
 our $UseHttp2 = $ENV{TEST_NGINX_USE_HTTP2};
 
 our $UseHup = $ENV{TEST_NGINX_USE_HUP};
@@ -39,12 +43,18 @@ our $UseRr = $ENV{TEST_NGINX_USE_RR};
 
 our $Verbose = $ENV{TEST_NGINX_VERBOSE};
 
+our $ld_preload = $ENV{LD_PRELOAD};
+
+our $ValgrindExitOnFirstErr = $ENV{TEST_NGINX_VALGRIND_EXIT_ON_FIRST_ERR};
+
 our $LatestNginxVersion = 0.008039;
 
 our $NoNginxManager = $ENV{TEST_NGINX_NO_NGINX_MANAGER} || 0;
 our $Profiling = 0;
 
+sub expand_env_in_text ($$$);
 sub use_http2 ($);
+sub use_http3 ($);
 
 our $InSubprocess;
 our $RepeatEach = 1;
@@ -66,6 +76,8 @@ our $PostponeOutput = $ENV{TEST_NGINX_POSTPONE_OUTPUT};
 
 our $Timeout = $ENV{TEST_NGINX_TIMEOUT} || 3;
 
+our $QuicIdleTimeout = $ENV{TEST_NGINX_QUIC_IDLE_TIMEOUT} || 0.6;
+
 our $CheckLeak = $ENV{TEST_NGINX_CHECK_LEAK} || 0;
 
 our $Benchmark = $ENV{TEST_NGINX_BENCHMARK} || 0;
@@ -78,8 +90,31 @@ our $ServerAddr = '127.0.0.1';
 
 our $ServerName = 'localhost';
 
+our $Http3SSLCrt = $ENV{TEST_NGINX_HTTP3_CRT};
+our $Http3SSLCrtKey = $ENV{TEST_NGINX_HTTP3_KEY};;
+
+our $ValgrindExtraTimeout = 0.3;
+
+if ($UseHttp2 && $UseHttp3) {
+    die "Ambiguous: both TEST_NGINX_USE_HTTP3 and TEST_NGINX_USE_HTTP2 are set.\n";
+}
+
+sub bail_out (@);
+
+if ($UseHttp3 && (!defined $Http3SSLCrt || !defined $Http3SSLCrtKey)) {
+    warn <<_EOC_;
+Please generate the certificate/key pair using the following command,
+and set TEST_NGINX_HTTP3_CRT and TEST_NGINX_HTTP3_KEY enviroment variables:
+  openssl req -x509 -newkey rsa:4096 -keyout http3.key -out http3.crt -days \\
+  7000 -nodes -subj '/CN=localhost' 2> /dev/null
+_EOC_
+    die "certificate/key pair not found";
+}
+
+our $ServerConfigHttp3 = '';
+
 our $WorkerUser = $ENV{TEST_NGINX_WORKER_USER};
-if (defined $WorkerUser && $WorkerUser !~ /^\w+(?:\s+\w+)?$/) {
+if (defined $WorkerUser && $WorkerUser !~ /^\w[-\w]+(?:\s+\w[-\w]+)?$/) {
     die "Bad value in the env TEST_NGINX_WORKER_USER: $WorkerUser\n";
 }
 
@@ -183,7 +218,9 @@ sub gen_rand_port (;$$) {
     my $rand_port;
 
     for (my $i = 0; $i < $tries; $i++) {
-        my $port = int(rand 63550) + 1985;
+        # NB: reserved ports for stream_server_config* (1..3)
+        # 1984 + 3 + 1 = 1988
+        my $port = int(rand 63547) + 1988;
 
         next if $used_ports->{$port};
 
@@ -191,7 +228,9 @@ sub gen_rand_port (;$$) {
             LocalAddr => $ServerAddr,
             LocalPort => $port,
             Proto => 'tcp',
+            Listen => 5,
             Timeout => 0.1,
+            Reuse => 1,
         );
 
         if (defined $sock) {
@@ -208,6 +247,44 @@ sub gen_rand_port (;$$) {
     return $rand_port;
 }
 
+sub is_udp_port_used($) {
+    my $port = shift;
+    my $filename = "/proc/net/udp";
+
+    open my $fh, $filename or die "Could not open $filename. $!";
+    while (<$fh>) {
+        my $line = $_;
+        if ($line =~ /^ *\d+: [0-9A-F]+:([0-9A-F]+) /) {
+            if ($port == hex($1)) {
+                close $fh;
+                return 1;
+            }
+        }
+    }
+
+    close $fh;
+    return 0;
+}
+
+sub is_tcp_port_used($) {
+    my $port = shift;
+    my $filename = "/proc/net/tcp";
+
+    open my $fh, $filename or die "Could not open $filename. $!";
+    while (<$fh>) {
+        my $line = $_;
+        if ($line =~ /^ *\d+: [0-9A-F]+:([0-9A-F]+) /) {
+            if ($port == hex($1)) {
+                close $fh;
+                return 1;
+            }
+        }
+    }
+
+    close $fh;
+    return 0;
+}
+
 sub no_long_string () {
     $NoLongString = 1;
 }
@@ -276,14 +353,13 @@ sub use_hup() {
 }
 
 our @CleanupHandlers;
+our @TestCleanupHandlers;
 our @BlockPreprocessors;
 
-sub bail_out (@);
-
 our $Randomize              = $ENV{TEST_NGINX_RANDOMIZE};
 our $NginxBinary            = $ENV{TEST_NGINX_BINARY} || 'nginx';
 our $Workers                = 1;
-our $WorkerConnections      = 64;
+our $WorkerConnections      = $ENV{TEST_NGINX_USE_HTTP3} ? 1024 : 64;
 our $LogLevel               = $ENV{TEST_NGINX_LOG_LEVEL} || 'debug';
 our $MasterProcessEnabled   = $ENV{TEST_NGINX_MASTER_PROCESS} || 'off';
 our $DaemonEnabled          = 'on';
@@ -417,7 +493,9 @@ sub master_process_enabled (@) {
 }
 
 our @EXPORT = qw(
+    expand_env_in_text
     use_http2
+    use_http3
     env_to_nginx
     is_str
     check_accum_error_log
@@ -436,9 +514,11 @@ our @EXPORT = qw(
     stap_out_fname
     bail_out
     add_cleanup_handler
+    add_test_cleanup_handler
     access_log_data
     error_log_data
     setup_server_root
+    run_customer_init_script
     write_config_file
     get_canon_version
     get_nginx_version
@@ -450,6 +530,7 @@ our @EXPORT = qw(
     $ServerPortForClient
     $ServerPort
     $NginxVersion
+    $PcreVersion
     $PidFile
     $ServRoot
     $ConfFile
@@ -480,6 +561,8 @@ our @EXPORT = qw(
     server_port_for_client
     no_nginx_manager
     use_hup
+    is_udp_port_used
+    is_tcp_port_used
 );
 
 
@@ -501,6 +584,7 @@ our $CheckErrorLog;
 our $CheckShutdownErrorLog;
 
 our $NginxVersion;
+our $PcreVersion;
 our $NginxRawVersion;
 our $OpenSSLVersion;
 
@@ -528,6 +612,7 @@ our $HtmlDir    = File::Spec->catfile($ServRoot, 'html');
 our $ConfDir    = File::Spec->catfile($ServRoot, 'conf');
 our $ConfFile   = File::Spec->catfile($ConfDir, 'nginx.conf');
 our $PidFile    = File::Spec->catfile($LogDir, 'nginx.pid');
+our $CacheDir   = File::Spec->catfile($ServRoot, 'cache');
 
 sub parse_time ($) {
     my $tm = shift;
@@ -557,6 +642,10 @@ sub add_cleanup_handler ($) {
    unshift @CleanupHandlers, shift;
 }
 
+sub add_test_cleanup_handler ($) {
+   unshift @TestCleanupHandlers, shift;
+}
+
 sub bail_out (@) {
     cleanup();
     Test::More::BAIL_OUT(@_);
@@ -566,28 +655,43 @@ sub kill_process ($$$) {
     my ($pid, $wait, $name) = @_;
 
     if ($wait) {
-        eval {
-            if (defined $pid) {
+        if (defined $pid) {
+            if ($ENV{TEST_NGINX_FAST_SHUTDOWN}) {
+                if ($Verbose) {
+                    warn "sending TERM signal to $pid";
+                }
+
+                kill(SIGTERM, $pid);
+
+            } else {
                 if ($Verbose) {
                     warn "sending QUIT signal to $pid";
                 }
 
                 kill(SIGQUIT, $pid);
             }
+        }
 
-            if ($Verbose) {
-                warn "waitpid timeout: ", timeout();
-            }
+        if ($Verbose) {
+            warn "waitpid timeout: ", timeout();
+        }
 
-            local $SIG{ALRM} = sub { die "alarm\n" };
-            alarm timeout();
-            waitpid($pid, 0);
-            alarm 0;
-        };
+        my $timeout_val = timeout();
+        while ($timeout_val > 0 && is_running($pid)) {
+            waitpid($pid, WNOHANG);
+            sleep 0.05;
+            $timeout_val -= 0.05;
+        }
 
-        if ($@) {
-            if ($Verbose) {
-                warn "$name - WARNING: child process $pid timed out.\n";
+        if (is_running($pid)) {
+            warn "$name - timeout when waiting for the process $pid to exit";
+            if (getpgrp($pid) == $pid) {
+                kill(SIGKILL, -$pid);
+                sleep 0.05;
+
+            } else {
+                kill(SIGKILL, $pid);
+                waitpid($pid, 0);
             }
         }
     }
@@ -621,8 +725,15 @@ sub kill_process ($$$) {
     #system("ps aux|grep $pid > /dev/stderr");
     warn "$name - WARNING: killing the child process $pid with force...";
 
-    kill(SIGKILL, $pid);
-    waitpid($pid, 0);
+    if (getpgrp($pid) == $pid) {
+        kill(SIGKILL, -$pid);
+        sleep 0.05;
+
+    } else {
+        kill(SIGKILL, $pid);
+        waitpid($pid, 0);
+    }
+
 
     if (is_running($pid)) {
         local $SIG{ALRM} = sub { die "alarm\n" };
@@ -632,6 +743,17 @@ sub kill_process ($$$) {
     }
 }
 
+sub cleanup_test ($) {
+    my $block = shift;
+    if ($Verbose) {
+        warn "cleaning up test ", $block->name;
+    }
+
+    for my $hdl (@TestCleanupHandlers) {
+       $hdl->($block);
+    }
+}
+
 sub cleanup () {
     if ($Verbose) {
         warn "cleaning up everything";
@@ -705,7 +827,9 @@ sub check_prev_block_shutdown_error_log () {
     my $name = $block->name;
     my $dry_run;
 
-    if (defined $block->shutdown_error_log) {
+    if (defined($block->shutdown_error_log)
+        || defined($block->no_shutdown_error_log))
+    {
         if ($UseHup) {
             $dry_run = 1;
         }
@@ -725,6 +849,8 @@ sub run_tests () {
         #warn "[INFO] Using nginx version $NginxVersion ($NginxRawVersion)\n";
     }
 
+    $PcreVersion = get_pcre_version();
+
     if (!defined $ENV{TEST_NGINX_SERVER_PORT}) {
         $ENV{TEST_NGINX_SERVER_PORT} = $ServerPort;
     }
@@ -734,7 +860,9 @@ sub run_tests () {
             $hdl->($block);
         }
 
+        $block->set_value("name", $0 . " " . $block->name);
         run_test($block);
+        cleanup_test($block);
 
         $PrevBlock = $block;
     }
@@ -765,7 +893,7 @@ sub setup_server_root ($) {
             }}, $ServRoot);
 
         } else {
-            remove_tree($HtmlDir, $LogDir);
+            remove_tree($CacheDir, $HtmlDir, $LogDir);
             rmdir $ServRoot or
                 bail_out "Can't remove $ServRoot (not empty?): $!";
         }
@@ -796,6 +924,18 @@ sub setup_server_root ($) {
         bail_out "Failed to do mkdir $ConfDir\n";
 }
 
+sub run_customer_init_script ($) {
+    my $block = shift;
+    my $name = $block->name;
+
+    if ($block->post_setup_server_root) {
+        eval $block->post_setup_server_root;
+        if ($@) {
+            bail_out("$name - post_setup_server_root failed: $@");
+        }
+    }
+}
+
 sub write_user_files ($$) {
     my ($block, $rand_ports) = @_;
 
@@ -856,7 +996,7 @@ sub write_user_files ($$) {
                 }
             }
 
-            $body = expand_env_in_text($body, $name, $rand_ports);
+            $body = expand_env_in_text $body, $name, $rand_ports;
 
             open my $out, ">$path" or
                 bail_out "$name - Cannot open $path for writing: $!\n";
@@ -894,7 +1034,7 @@ sub write_config_file ($$$) {
         master_off();
     }
 
-    $http_config = expand_env_in_text($http_config, $name, $rand_ports);
+    $http_config = expand_env_in_text $http_config, $name, $rand_ports;
 
     if (!defined $config) {
         $config = '';
@@ -928,17 +1068,17 @@ sub write_config_file ($$$) {
     if ($LoadModules) {
         my @modules = map { "load_module $_;" } grep { $_ } split /\s+/, $LoadModules;
         if (@modules) {
-            $main_config .= join " ", @modules;
+            $main_config = join " ", @modules, $main_config;
         }
     }
 
-    $main_config = expand_env_in_text($main_config, $name, $rand_ports);
+    $main_config = expand_env_in_text $main_config, $name, $rand_ports ;
 
     if (!defined $post_main_config) {
         $post_main_config = '';
     }
 
-    $post_main_config = expand_env_in_text($post_main_config, $name, $rand_ports);
+    $post_main_config = expand_env_in_text $post_main_config, $name, $rand_ports;
 
     if ($CheckLeak || $Benchmark) {
         $LogLevel = 'warn';
@@ -949,6 +1089,8 @@ sub write_config_file ($$$) {
         $err_log_file = $ErrLogFile;
     }
 
+    $err_log_file = expand_env_in_text $err_log_file, $name, $rand_ports;
+
     if (!defined $server_name) {
         $server_name = $ServerName;
     }
@@ -981,15 +1123,77 @@ _EOC_
     }
 
     my $listen_opts = '';
+    # To maintain compatibility with old test cases, the $http2_directive
+    # and listen directive need to be placed on the same line.
+    my $http2_directive = '';
 
-    if (use_http2($block)) {
-        $listen_opts .= " http2";
+    $ServerConfigHttp3 = '';
+    if (use_http3($block)) {
+        if ($UseHttp3 && !defined $block->http3) {
+            $ServerConfigHttp3 = "ssl_protocols TLSv1.3;";
+
+            if (defined $Http3SSLCrt) {
+                $ServerConfigHttp3 .= "ssl_certificate $Http3SSLCrt;";
+            }
+
+            if (defined $Http3SSLCrtKey) {
+                $ServerConfigHttp3 .= "ssl_certificate_key $Http3SSLCrtKey;";
+            }
+        }
+
+    } elsif (use_http2($block)) {
+        if ($NginxVersion >= "1.019009") {
+            $http2_directive = "http2 on;";
+        } else {
+            $listen_opts .= " http2";
+        }
     }
 
     if ($ReusePort) {
         $listen_opts .= " reuseport";
     }
 
+    my $keepalive_timeout = 68000;
+    if (use_http3($block)) {
+        my $keepalive_timeout_sec = $QuicIdleTimeout;
+        if ($block->quic_max_idle_timeout) {
+            $keepalive_timeout_sec = $block->quic_max_idle_timeout;
+        }
+
+        if ($UseValgrind) {
+            $keepalive_timeout_sec += $ValgrindExtraTimeout;
+        }
+
+        $keepalive_timeout = int($keepalive_timeout_sec * 1000);
+    }
+
+    my $nginx_V = `$NginxBinary -V 2>&1`;
+    my $fastcgi_temp_path = "fastcgi_temp_path $ServRoot/fastcgi_temp;";
+    my $scgi_temp_path = "scgi_temp_path $ServRoot/scgi_temp;";
+    my $uwsgi_temp_path = "uwsgi_temp_path $ServRoot/uwsgi_temp;";
+    my $proxy_temp_path = "proxy_temp_path $ServRoot/proxy_temp;";
+    my $client_body_temp_path = "client_body_temp_path $ServRoot/client_body_temp;";
+
+    if ($nginx_V =~ /--without-http_fastcgi_module/) {
+        $fastcgi_temp_path = '';
+    }
+
+    if ($nginx_V =~ /--without-http_scgi_module/) {
+        $scgi_temp_path = '';
+    }
+
+    if ($nginx_V =~ /--without-http_uwsgi_module/) {
+        $uwsgi_temp_path = '';
+    }
+
+    if ($http_config =~ /\bproxy_temp_path\b/) {
+        $proxy_temp_path ='';
+    }
+
+    if ($http_config =~ /\bclient_body_temp_path\b/) {
+        $client_body_temp_path ='';
+    }
+
     print $out <<_EOC_;
 #env LUA_PATH;
 #env LUA_CPATH;
@@ -997,18 +1201,36 @@ _EOC_
 $main_config
 
 http {
-    access_log $AccLogFile;
+    access_log $AccLogFile; $client_body_temp_path $proxy_temp_path $fastcgi_temp_path $scgi_temp_path $uwsgi_temp_path
     #access_log off;
 
     default_type text/plain;
-    keepalive_timeout  68;
+    keepalive_timeout  ${keepalive_timeout}ms;
 
 $http_config
-
     server {
-        listen          $ServerPort$listen_opts;
-        server_name     '$server_name';
+        listen          $ServerPort$listen_opts;$http2_directive
+_EOC_
 
+    # when using http3, we both listen on tcp for http and udp for http3
+    if (use_http3($block)) {
+        my $h3_listen_opts = $listen_opts;
+        if ($h3_listen_opts !~ /\breuseport\b/) {
+            $h3_listen_opts .= " reuseport";
+        }
+
+        print $out <<_EOC_;
+        listen          $ServerPort$h3_listen_opts quic;
+_EOC_
+    } else {
+        print $out <<_EOC_;
+        #placeholder
+_EOC_
+    }
+
+    print $out <<_EOC_;
+        server_name     '$server_name';
+        $ServerConfigHttp3
         client_max_body_size 30M;
         #client_body_buffer_size 4k;
 
@@ -1036,7 +1258,7 @@ _EOC_
     if ($UseHup) {
         print $out <<_EOC_;
     server {
-        listen          $ServerPort$listen_opts;
+        listen          $ServerPort$listen_opts; $http2_directive
         server_name     'Test-Nginx';
 
         location = /ver {
@@ -1061,6 +1283,16 @@ events {
     worker_connections  $WorkerConnections;
 _EOC_
 
+    if (defined $block->no_mockeagain) {
+        if (defined $ld_preload) {
+            my $new_ld_preload = $ld_preload;
+            $new_ld_preload =~ s/[^ ]+mockeagain.so//;
+            $ENV{LD_PRELOAD} = $new_ld_preload;
+        }
+    } else {
+        $ENV{LD_PRELOAD} = $ld_preload;
+    }
+
     if ($EventType) {
         print $out <<_EOC_;
     use $EventType;
@@ -1104,11 +1336,11 @@ sub get_nginx_version () {
         bail_out("Failed to get the version of the Nginx in PATH: $out");
     }
 
-    if ($out =~ m{built with OpenSSL (\d+)\.(\d+)\.(\d+)([a-z])}s) {
+    if ($out =~ m{built with OpenSSL (\d+)\.(\d+)\.(\d+)([a-z])?}s) {
         $OpenSSLVersion = get_canon_version_for_OpenSSL($1, $2, $3, $4);
     }
 
-    if ($out =~ m{(?:nginx|openresty)[^/]*/(\d+)\.(\d+)\.(\d+)}s) {
+    if ($out =~ m{(?:nginx|openresty+?)[^/]*/(\d+)\.(\d+)\.(\d+)}s) {
         $NginxRawVersion = "$1.$2.$3";
         return get_canon_version($1, $2, $3);
     }
@@ -1121,6 +1353,34 @@ sub get_nginx_version () {
     bail_out("Failed to parse the output of \"nginx -V\": $out\n");
 }
 
+sub get_pcre_version () {
+    my $out = `$NginxBinary -V 2>&1`;
+    if (!defined ($out) || $? != 0) {
+        $out //= "";
+        bail_out("Failed to get the pcre version of the Nginx in PATH: $out");
+    }
+
+    my $ver = 0;
+    my $without_pcre2 = 0;
+    if ($out =~ /nginx version: .*\/(\d)\.(\d+)(?:\.\d+)+/) {
+        $ver = $1 * 100 + $2;
+    }
+
+    if ($out =~ /--without-pcre2/) {
+        $without_pcre2 = 1;
+    }
+
+    if ($ver >= 125 && $without_pcre2 == 0) {
+        return 2;
+
+    } elsif ($out =~ /pcre/) {
+        return 1;
+
+    } else {
+        return 0;
+    }
+}
+
 sub get_pid_from_pidfile ($) {
     my ($name) = @_;
 
@@ -1161,12 +1421,15 @@ sub test_config_version ($$) {
     for (my $tries = 1; $tries <= $total; $tries++) {
 
         my $extra_curl_opts = '';
+        my $http_protocol = "http";
 
         if (use_http2($block)) {
-            $extra_curl_opts .= ' --http2 --http2-prior-knowledge';
+            $extra_curl_opts .= ' --http2-prior-knowledge';
         }
 
-        my $cmd = "curl$extra_curl_opts -sS -H 'Host: Test-Nginx' --connect-timeout 2 'http://$ServerAddr:$ServerPort/ver'";
+        #server Test-Nginx only listen on http(tcp port) when http3 is enabled
+
+        my $cmd = "curl$extra_curl_opts -sS -H 'Host: Test-Nginx' --connect-timeout 2 '$http_protocol://$ServerAddr:$ServerPort/ver'";
         #warn $cmd;
         my $ver = `$cmd`;
         #chop $ver;
@@ -1359,11 +1622,33 @@ sub run_test ($) {
         undef $FirstTime;
     }
 
+    # add test case name to the asan log name.
+    if (defined($ENV{ASAN_OPTIONS})) {
+        if (!defined $ENV{ORI_ASAN_OPTIONS}) {
+            $ENV{ORI_ASAN_OPTIONS} = $ENV{ASAN_OPTIONS};
+        }
+        my $asan_opts = $ENV{ORI_ASAN_OPTIONS};
+        my $tag = $name;
+        if ($tag =~ /t\/([-_a-z0-9A-Z\/]+)\.t TEST (\d+):/) {
+            $tag = "$1-t$2";
+            $tag =~ s#/#-#g;
+            if ($asan_opts =~ /log_path=([^,]+)/) {
+                my $log_path = $1;
+                $asan_opts =~ s/log_path=[^,]+,?//;
+                $asan_opts .= ",log_path=$log_path-$tag";
+                $ENV{ASAN_OPTIONS} = $asan_opts;
+            }
+
+        } else {
+            warn "unknown log name: $tag";
+        }
+    }
     my $rand_ports = {};
+    $RandPorts = $rand_ports;
 
     my $config = $block->config;
 
-    $config = expand_env_in_text($config, $name, $rand_ports);
+    $config = expand_env_in_text $config, $name, $rand_ports;
 
     my $dry_run = 0;
     my $should_restart = 1;
@@ -1676,6 +1961,7 @@ sub run_test ($) {
                     }
 
                     setup_server_root($first_time);
+                    run_customer_init_script($block);
                     write_user_files($block, $rand_ports);
                     write_config_file($block, $config, $rand_ports);
 
@@ -1692,6 +1978,34 @@ sub run_test ($) {
                         if (system("kill -HUP $pid") == 0) {
                             sleep $TestNginxSleep * 3;
 
+                            # wait for http3 connections to timeout
+                            # so older nginx can exit
+                            if (use_http3($block)) {
+                                my $idle_time = $QuicIdleTimeout;
+                                if ($block->quic_max_idle_timeout) {
+                                    $idle_time = $block->quic_max_idle_timeout;
+                                }
+
+                                if ($UseValgrind) {
+                                    $idle_time += $ValgrindExtraTimeout;
+                                }
+
+                                sleep (0.1 + $idle_time);
+                                my $remain = 1.0;
+                                while ($remain >= 0) {
+                                    my $shutting = `pgrep -P $pid | xargs -n1 ps --noheader -o cmd -p | grep shutting`;
+                                    if ($shutting eq "") {
+                                        last;
+                                    }
+
+                                    $remain -= 0.1;
+                                }
+
+                                if ($remain <= 0.0) {
+                                    warn "$name - nginx shutting down timeout.\n";
+                                }
+                            }
+
                             if ($Verbose) {
                                 warn "skip starting nginx from scratch\n";
                             }
@@ -1754,6 +2068,7 @@ start_nginx:
 
             #warn "*** Restarting the nginx server...\n";
             setup_server_root($first_time);
+            run_customer_init_script($block);
             write_user_files($block, $rand_ports);
             write_config_file($block, $config, $rand_ports);
             #warn "nginx binary: $NginxBinary";
@@ -1772,11 +2087,38 @@ start_nginx:
             my $cmd;
 
             if ($NginxVersion >= 0.007053) {
-                $cmd = "$NginxBinary -p $ServRoot/ -c $ConfFile > /dev/null";
+                $cmd = "$NginxBinary -e $ServRoot/logs/error.log -p $ServRoot/ -c $ConfFile > /dev/null";
             } else {
                 $cmd = "$NginxBinary -c $ConfFile > /dev/null";
             }
 
+            my $LD_PRELOAD = $ENV{LD_PRELOAD};
+            if (defined $ENV{TEST_NGINX_LD_PRELOAD}) {
+                if (defined $LD_PRELOAD) {
+                    $LD_PRELOAD = "$ENV{TEST_NGINX_LD_PRELOAD} $LD_PRELOAD";
+                } else {
+                    $LD_PRELOAD = $ENV{TEST_NGINX_LD_PRELOAD};
+                }
+
+                $cmd = qq!LD_PRELOAD="$LD_PRELOAD" $cmd!;
+            }
+
+            my $mockeagain = $ENV{MOCKEAGAIN};
+            if (defined($mockeagain) && $mockeagain ne ""
+                && defined($LD_PRELOAD) && $LD_PRELOAD =~ /mockeagain.so/)
+            {
+                my $t = $ENV{TEST_NGINX_EVENT_TYPE};
+                my $pp = $ENV{TEST_NGINX_POSTPONE_OUTPUT};
+
+                if (!defined($t) || $t ne "poll") {
+                    warn "Warning: $name TEST_NGINX_EVENT_TYPE should be poll\n";
+                }
+            }
+
+            if (defined $block->suppress_stderr) {
+                $cmd .= " 2> /dev/null";
+            }
+
             if ($UseRr) {
                 $cmd = "rr record $cmd";
             }
@@ -1785,7 +2127,17 @@ start_nginx:
                 my $opts;
 
                 if ($UseValgrind =~ /^\d+$/) {
-                    $opts = "--tool=memcheck --leak-check=full --show-possibly-lost=no";
+                    $opts = "--tool=memcheck --leak-check=full --keep-debuginfo=yes --show-possibly-lost=no";
+                    if ($ValgrindExitOnFirstErr) {
+                        my $help_out = `valgrind --help`;
+                        if ($help_out =~ /exit-on-first-error/) {
+                            $opts .= " --exit-on-first-error=yes --error-exitcode=1";
+
+                        } else {
+                            warn "WARNING: valgrind does not support "
+                               . "--exit-on-first-error option\n";
+                        }
+                    }
 
                     if (-f 'valgrind.suppress') {
                         $cmd = "valgrind --num-callers=100 -q $opts --gen-suppressions=all --suppressions=valgrind.suppress $cmd";
@@ -1897,6 +2249,7 @@ start_nginx:
                     # child process
                     #my $rc = system($cmd);
 
+                    setpgrp;
                     my $tb = Test::More->builder;
                     $tb->no_ending(1);
 
@@ -2023,7 +2376,7 @@ request:
         #warn "Use hup: $UseHup, i: $i\n";
 
         if ($Verbose) {
-            warn "Run the test block...\n";
+            warn "Run the test block ", $block->name, " ...\n";
         }
 
         if (($CheckLeak || $Benchmark) && defined $block->tcp_listen) {
@@ -2044,6 +2397,8 @@ request:
 
             my $target = $block->tcp_listen;
 
+            $target = expand_env_in_text $target, $name, $rand_ports;
+
             my $reply = $block->tcp_reply;
             if (!defined $reply && !defined $block->tcp_shutdown) {
                 bail_out("$name - no --- tcp_reply specified but --- tcp_listen is specified");
@@ -2180,8 +2535,26 @@ request:
                         warn "TCP server reading request...\n";
                     }
 
+                    my $timeout = 0.1;
+                    my $select = IO::Select->new($client);
+                    if (defined($block->tcp_query_auto_timeout)) {
+                        $client->blocking(0);
+                        if ($block->tcp_query_auto_timeout ne "") {
+                            $timeout = $block->tcp_query_auto_timeout + 0;
+                        }
+                    }
+
                     while (1) {
                         my $b;
+                        if (defined($block->tcp_query_auto_timeout)) {
+                            if (!$select->can_read($timeout)) {
+                                if ($Verbose) {
+                                    warn "tcp_query read timeout\n";
+                                }
+                                last;
+                            }
+                        }
+
                         my $ret = $client->recv($b, 4096);
                         if (!defined $ret) {
                             die "failed to receive: $!\n";
@@ -2197,24 +2570,35 @@ request:
                         # flush read data to the file as soon as possible:
 
                         if ($tcp_query_file) {
-                            open my $out, ">$tcp_query_file"
-                                or die "cannot open $tcp_query_file for writing: $!\n";
+                            my $tmpfile = "$tcp_query_file.tmp";
+                            open my $out, ">$tmpfile"
+                                or die "cannot open $tmpfile for writing: $!\n";
 
                             if ($Verbose) {
-                                warn "writing received data [$buf] to file $tcp_query_file\n";
+                                warn "writing received data [$buf] to file $tmpfile\n";
                             }
 
                             print $out $buf;
                             close $out;
+                            # rename is atomic on Unix
+                            rename $tmpfile, $tcp_query_file
+                                or die "cannot rename $tmpfile to $tcp_query_file: $!\n";
                         }
 
-                        if (!$req_len || length($buf) >= $req_len) {
+                        if (!defined($block->tcp_query_auto_timeout) && (!$req_len || length($buf) >= $req_len)) {
                             if ($Verbose) {
+                                $req_len //= 0;
                                 warn "len: ", length($buf), ", req len: $req_len\n";
                             }
                             last;
                         }
                     }
+
+                    if (!$tcp_query_file) {
+                        if ($Verbose) {
+                            warn "received data [$buf]\n";
+                        }
+                    }
                 }
 
                 my $delay = parse_time($block->tcp_reply_delay);
@@ -2310,6 +2694,9 @@ request:
             }
 
             my $target = $block->udp_listen;
+
+            $target = expand_env_in_text $target, $name, $rand_ports;
+
             if ($target =~ /^\d+$/) {
                 my $port = $target;
 
@@ -2377,15 +2764,19 @@ request:
                 }
 
                 if ($udp_query_file) {
-                    open my $out, ">$udp_query_file"
-                        or die "cannot open $udp_query_file for writing: $!\n";
+                    my $tmpfile = "$udp_query_file.tmp";
+                    open my $out, ">$tmpfile"
+                        or die "cannot open $tmpfile for writing: $!\n";
 
                     if ($Verbose) {
-                        warn "writing received data [$buf] to file $udp_query_file\n";
+                        warn "writing received data [$buf] to file $tmpfile\n";
                     }
 
                     print $out $buf;
                     close $out;
+                    # rename is atomic on UNIX
+                    rename $tmpfile, $udp_query_file
+                        or die "cannot rename $tmpfile to $udp_query_file: $!\n";
                 }
 
                 my $delay = parse_time($block->udp_reply_delay);
@@ -2555,7 +2946,8 @@ retry:
                         warn "sending KILL signal to $pid";
                     }
 
-                    kill(SIGKILL, $pid);
+                    # kill process group, including children
+                    kill(SIGKILL, -$pid);
                     waitpid($pid, 0);
 
                     if (!unlink($PidFile) && -f $PidFile) {
@@ -2564,6 +2956,19 @@ retry:
 
                 } else {
                     #warn "nginx killed";
+                    waitpid($pid, WNOHANG);
+                    my $timeout_val = 1.0;
+                    while ($timeout_val > 0 && is_running($pid)) {
+                        waitpid($pid, WNOHANG);
+                        sleep 0.05;
+                        $timeout_val -= 0.05;
+                    }
+
+                    if (is_running($pid)) {
+                        warn "$name - timeout when waiting for the process $pid to exit";
+                        kill(SIGKILL, $pid);
+                        sleep 0.05;
+                    }
                 }
 
             } else {
@@ -2638,6 +3043,10 @@ sub use_http2 ($) {
         return $cached;
     }
 
+    if (defined $block->no_http2) {
+        return undef;
+    }
+
     if (defined $block->http2) {
         if ($block->raw_request) {
             bail_out("cannot use --- http2 with --- raw_request");
@@ -2680,6 +3089,41 @@ sub use_http2 ($) {
             return undef;
         }
 
+        if (defined $block->http3) {
+            warn "WARNING: ", $block->name, " - explicitly requires HTTP/3, so will not use HTTP/2\n";
+            $block->set_value("test_nginx_enabled_http2", 0);
+            return undef;
+        }
+
+        my $pat = qr{(proxy_pass .*:\$(server_port|TEST_NGINX_SERVER_PORT)
+                    | ngx.req.raw_header
+                    | lua_check_client_abort
+                    | ngx.req.socket
+                    | ngx.req.read_body)}x;
+        if (defined $block->config) {
+            if ($block->config =~ $pat) {
+                warn "WARNING: ", $block->name, " - $1 does not support in HTTP/2\n";
+                $block->set_value("test_nginx_enabled_http2", 0);
+                return undef;
+            }
+        }
+
+        if (defined $block->user_files) {
+            if ($block->user_files =~ $pat) {
+                warn "WARNING: ", $block->name, " - $1 does not support HTTP/2\n";
+                $block->set_value("test_nginx_enabled_http2", 0);
+                return undef;
+            }
+        }
+
+        if (defined $block->http_config) {
+            if ($block->http_config=~ $pat) {
+                warn "WARNING: ", $block->name, " - can not listen on HTTP and HTTP/2 at the same time, so will no use HTTP2\n";
+                $block->set_value("test_nginx_enabled_http2", 0);
+                return undef;
+            }
+        }
+
         $block->set_value("test_nginx_enabled_http2", 1);
 
         if (!$LoadedIPCRun) {
@@ -2694,4 +3138,83 @@ sub use_http2 ($) {
     return undef;
 }
 
+
+sub use_http3 ($) {
+    my $block = shift;
+    my $cached = $block->test_nginx_enabled_http3;
+
+    if (defined $cached) {
+        return $cached;
+    }
+
+    if (defined $block->no_http3) {
+        return undef;
+    }
+
+    if (defined $block->http3) {
+        if ($block->raw_request) {
+            bail_out("cannot use --- http3 with --- raw_request");
+        }
+
+        if ($block->pipelined_requests) {
+            bail_out("cannot use --- http3 with --- pipelined_requests");
+        }
+
+        if (defined $block->http2) {
+            bail_out("cannot use --- http3 with --- http2");
+        }
+
+        $block->set_value("test_nginx_enabled_http3", 1);
+
+        if (!$LoadedIPCRun) {
+            require IPC::Run;
+            $LoadedIPCRun = 1;
+        }
+
+        return 1;
+    }
+
+    if ($UseHttp3) {
+        if ($block->raw_request) {
+            warn "WARNING: ", $block->name, " - using raw_request HTTP/2, will not use HTTP/3\n";
+            $block->set_value("test_nginx_enabled_http3", 0);
+            return undef;
+        }
+
+        if ($block->pipelined_requests) {
+            warn "WARNING: ", $block->name, " - using pipelined_requests, will not use HTTP/3\n";
+            $block->set_value("test_nginx_enabled_http3", 0);
+            return undef;
+        }
+
+        if (!defined $block->request) {
+            $block->set_value("test_nginx_enabled_http3", 0);
+            return undef;
+        }
+
+        if (!ref $block->request && $block->request =~ m{HTTP/1\.0}s) {
+            warn "WARNING: ", $block->name, " - explicitly requires HTTP 1.0, so will not use HTTP/3\n";
+            $block->set_value("test_nginx_enabled_http3", 0);
+            return undef;
+        }
+
+        if (defined $block->http2) {
+            warn "WARNING: ", $block->name, " - explicitly requires HTTP/2, so will not use HTTP/3\n";
+            $block->set_value("test_nginx_enabled_http3", 0);
+            return undef;
+        }
+
+        $block->set_value("test_nginx_enabled_http3", 1);
+
+        if (!$LoadedIPCRun) {
+            require IPC::Run;
+            $LoadedIPCRun = 1;
+        }
+        return 1;
+    }
+
+    $block->set_value("test_nginx_enabled_http3", 0);
+    return undef;
+}
+
 1;
